// tools.js

const
    CustomException = require('./custom-exception');

class Tools {

    static coerceClass(className) {
        //Return class for className, or null if not found
        if (typeof className === 'function')
            return className;
        let cl = null;
        try {
            const fct = new Function(`return ${className}`);
            cl = fct(className);
        } catch (ex) {
            //not found, nop
        }
        return cl;
    }


    static isStringValidNumber(str) {
        if (str.match(/^-?\d+$/))
            return true;
        return str.match(/^\d+\.\d+$/);
    }

    static isString(o) {
        return typeof o === 'string';
    }

    //-----------------------------------------------------------
    //Coercing

    static coerceNum(o) {
        if (!this.isString(o))
            return o;
        //it is a string
        if (!this.isStringValidNumber(o))
                throw new CustomException('');
        return parseFloat(o);
    }

    static coerceNumWithDefault(o, defaultNum) {
        //if number is not valid use defaultNum
        if (!o) {
            return defaultNum;
        }
        if (this.isString(o) && !this.isStringValidNumber(o))
            return defaultNum;
        return this.coerceNum(o);
    }

    //-----------------------------------------------------------
    //Validating

    //static parseNumsString(anInput) {
    static coerceNumsString(anInput) {
        //Assume pre-validated
        //Allow either whitespace or comma delimiter
        if (!anInput)
            return [];
        const input = anInput.trim();
        //Handle special case first
        if (input.length === 0)
            return [];
        const strings = input.trim().split(/[\s,]+/);
        return strings.map(eaString => parseFloat(eaString));
    }

    static findInvalidNumberIn(numsString) {
        //validate a numbers string
        //Return null if okay
        const s = numsString.trim();
        if (s.length === 0) return null;
        const strings = s.split(/[\s,]+/);
        for (let each of strings)
            if (!this.isStringValidNumber(each))
                return each;
        return null;
    }

    //-----------------------------------------------------------
    //Helpers

    static className(o) {
        return o.constructor.name;
    }

    /**
     * @param {TestCase} o
     * @param {string} methodName
     */
    static implementedBySubclass(o, methodName) {
        //throw new CustomException(`Class "${className(o)}" must implement method "${methodName}"`);
        throw new CustomException();
    }

    static decimalCount(num) {
        if (Math.floor(num) === num)
            return 0;
        const s = num.toString();
        if (!s.includes('.'))
            return 0;
        return s.split('.')[1].length;
    }

    //-----------------------------------------------------------
    //Exception Helpers

    static exceptionToDisplayStrings(ex) {
        const strings = [];
        try {
            const lines = ex.stack.split('\n');
            //Assume 0 is info, 1 is location
            strings.push(lines[0]);
            const line2 = lines[1];
            const
                messageLoc = this.parseExceptionMessageLocation(line2),
                fileLoc = this.parseExceptionFileLocation(line2);
            strings.push(`(method=${messageLoc}) ${fileLoc}`);
        } catch (ex) {
            //nop
        }
        return strings;
    }

    static parseExceptionMessageLocation(stackLine) {
        //Expecting something like:
        //	"    at Counter.getCount (file:/
        let key, i, sub;
        key = 'at ';
        i = stackLine.indexOf(key);
        if (i === -1) return '';
        sub = stackLine.substring(i + key.length);
        key = ' ';
        i = sub.indexOf(key);
        if (i === -1) return '';
        return sub.substring(0, i);
    }

    static parseExceptionFileLocation(stackLine) {
        //Expecting something like (at end of string):
        //	/model.js:17:5)"
        let i, sub, fn, lineNo;
        i = stackLine.lastIndexOf('/') + 1;
        if (i === -1) return '';
        sub = stackLine.substring(i);
        fn = sub.substring(0, sub.indexOf(':'));
        lineNo = sub.substring(sub.indexOf(':') + 1);
        lineNo = lineNo.substring(0, lineNo.indexOf(':'));
        //foo.js (line 101)
        return `(file="${fn}") (line=${lineNo})`;
    }

}

//--------------------------------
// Exporting

exports.Tools = Tools;